#################################################################################
#
#    Copyright (C) 2006 Utah State University, All rights reserved.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__author__  = '''David Ray, David Wiley'''
__docformat__ = 'plaintext'
__version__   = '$ Revision 0.0 $'[11:-2]

import os.path
import sys
from StringIO import StringIO

from App.Common import package_home
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import manage_addTool
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from zExceptions import NotFound, BadRequest

from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
try:
    from Products.Archetypes.lib.register import listTypes
except ImportError:
    from Products.Archetypes.public import listTypes
from Products.Folksemantic.config import PROJECTNAME
from Products.Folksemantic.config import product_globals as GLOBALS

def install(self):
    """ External Method to install FolksemanticTools """
    
    
    portal_conf=getToolByName(self,'portal_controlpanel')
    portal_conf.registerConfiglet( 'portal_folksemantictools'
           , 'Bookmarklets'       
           , 'string:${portal_url}/folksemantic' 
           , ''                 # a condition   
           , 'Manage portal'    # access permission
           , 'Products'         # section to which the configlet should be added: 
                                #(Plone,Products,Members) 
           , 0                  # visibility
           , 'portal__folksemanticID'                                        
           , 'arrowRight.gif' # icon in control_panel, put your own icon in the 
                                          # /skins folder of your product and change 
                                          # 'site_icon.gif' to 'yourfile'
           , ''
           , None
           )

    
    out = StringIO()
    print >> out, "Installation log of %s:" % PROJECTNAME

    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    try:
        from Products.Folksemantic.config import DEPENDENCIES
    except:
        DEPENDENCIES = []
    portal = getToolByName(self,'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)
        get_transaction().commit(1)

    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    #autoinstall tools
    portal = getToolByName(self,'portal_url').getPortalObject()
    for t in ['FolksemanticTool']:
        try:
            portal.manage_addProduct[PROJECTNAME].manage_addTool(t)
        except BadRequest:
            # if an instance with the same name already exists this error will
            # be swallowed. Zope raises in an unelegant manner a 'Bad Request' error
            pass
        except:
            e = sys.exc_info()
            if e[0] != 'Bad Request':
                raise


    #hide tools in the navigation
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList) + \
                                  [toolname for toolname in ['portal_folksemantictool'] \
                                            if toolname not in navtreeProperties.idsNotToList]



    #adds action for Annorate
    at=getToolByName(self, 'portal_actions')
    ai=getToolByName(self, 'portal_actionicons')
    at.addAction('Annorate',
                 name='Annorate',
                 action="string:javascript:(function(){e=encodeURIComponent;w=window;d=document;l=location;u='http://www.annorate.com/annorations/new?u='+e(l.href)+'&t='+e(d.title);w.onerror=function(){l.href=u;};dgs=d.getSelection;gs=w.getSelection;c=gs?gs():dgs?dgs():d.selection?d.selection.createRange().text:'';if(c==''){de=d.getElementsByName('Description');c=de&&de[0]?c=de[0].content:''}u=u+'&c='+e(c);l.href=u;})()",
                 condition='',
                 permission='View',
                 category='document_actions')
    ai.addActionIcon('plone','Annorate','annorate.gif','Annorate', 0)

    #adds action for Make a Path
    at=getToolByName(self, 'portal_actions')
    ai=getToolByName(self, 'portal_actionicons')
    at.addAction('MakeaPath',
                 name='Make a Path',
                 action="string:javascript:(function(){e=encodeURIComponent;w=window;d=document;l=location;u='http://www.makeapath.com/paths/new?u='+e(l.href)+'&t='+e(d.title);w.onerror=function(){l.href=u;};dgs=d.getSelection;gs=w.getSelection;c=gs?gs():dgs?dgs():d.selection?d.selection.createRange().text:'';if(c==''){de=d.getElementsByName('Description');c=de&&de[0]?c=de[0].content:''}u=u+'&c='+e(c);l.href=u;})()",
                 condition='',
                 permission='View',
                 category='document_actions')
    ai.addActionIcon('plone','MakeaPath','MakeaPath.gif','Make a Path', 0)

    #adds action for Send To Wiki
    at=getToolByName(self, 'portal_actions')
    ai=getToolByName(self, 'portal_actionicons')
    at.addAction('Send2Wiki',
                 name='Send To Wiki',
                 action="string:javascript:e=encodeURIComponent;w=window;d=document;s='http://www.send2wiki.com/index.php/Special:Send2Wiki';location.href=s+'?u='+e(location)+'&t='+e(d.title);",
                 condition='',
                 permission='View',
                 category='document_actions')
    ai.addActionIcon('plone','Send2Wiki','Send2Wiki.gif','Send2Wiki', 0)



    
    # try to call a workflow install method
    # in 'InstallWorkflows.py' method 'installWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'installWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Install:'
        res = installWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow install'


    # try to call a custom install method
    # in 'AppInstall.py' method 'install'
    try:
        install = ExternalMethod('temp','temp',PROJECTNAME+'.AppInstall', 'install')
    except NotFound:
        install = None

    if install:
        print >>out,'Custom Install:'
        res = install(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom install'
    return out.getvalue()

def uninstall(self):
    out = StringIO()
    
    
    portal_conf=getToolByName(self,'portal_controlpanel')
    portal_conf.unregisterConfiglet('portal_folksemantic')

    # unhide tools
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList)
            for toolname in [toolname for toolname in ['portal_folksemantictool'] \
                                      if toolname not in navtreeProperties.idsNotToList]:
                if toolname in navtreeProperties.idsNotToList:
                    navtreeProperties.idsNotToList.remove(toolname)

    # try to call a workflow uninstall method
    # in 'InstallWorkflows.py' method 'uninstallWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'uninstallWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Uninstall:'
        res = uninstallWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow uninstall'

    # try to call a custom uninstall method
    # in 'AppInstall.py' method 'uninstall'
    try:
        uninstall = ExternalMethod('temp','temp',PROJECTNAME+'.AppInstall', 'uninstall')
    except:
        uninstall = None

    if uninstall:
        print >>out,'Custom Uninstall:'
        res = uninstall(self)
        if res:
            print >>out,res
        else:
            print >>out,'no output'
    else:
        print >>out,'no custom uninstall'

    return out.getvalue()
